package org.artifactory.repo.cleanup;

import org.artifactory.api.context.ArtifactoryContext;
import org.artifactory.api.context.ArtifactoryContextThreadBinder;
import org.artifactory.api.repo.RepositoryService;
import org.artifactory.fs.ItemInfo;
import org.artifactory.model.common.RepoPathImpl;
import org.artifactory.repo.RepoPath;
import org.artifactory.test.ArtifactoryHomeBoundTest;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import static org.artifactory.common.ConstantValues.folderPruningQuietPeriodSecs;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

public class FolderPruningServiceImplTest extends ArtifactoryHomeBoundTest {
    private static final String LIBS_RELEASES_LOCAL = "libs-releases-local";

    @Mock
    RepositoryService repositoryService;

    @Mock
    private ArtifactoryContext contextMock;

    @Mock
    private ItemInfo itemInfo;

    @InjectMocks
    private FolderPruningServiceImpl folderPruningService;

    private AutoCloseable mocks;

    @BeforeMethod
    public void setup() {
        mocks = MockitoAnnotations.openMocks(this);
        ArtifactoryContextThreadBinder.bind(contextMock);
        when(contextMock.beanForType(RepositoryService.class)).thenReturn(repositoryService);
        homeStub.setProperty(folderPruningQuietPeriodSecs, "0");
    }

    @AfterMethod
    public void tearDown() throws Exception {
        ArtifactoryContextThreadBinder.unbind();
        mocks.close();
    }

    @Test
    public void testDontAddSameParentTwiceTest() {

        RepoPath fileRepoPath1 = new RepoPathImpl(LIBS_RELEASES_LOCAL, "radiohead/kid_a.txt");
        RepoPath fileRepoPath2 = new RepoPathImpl(LIBS_RELEASES_LOCAL, "radiohead/amnesiac.txt");

        when(repositoryService.getItemInfo(fileRepoPath1.getParent())).thenReturn(itemInfo);
        when(itemInfo.isFolder()).thenReturn(true);
        folderPruningService.prune(fileRepoPath1.getParent());
        folderPruningService.prune(fileRepoPath2.getParent());
        folderPruningService.doPrune();
        verify(repositoryService, times(1)).getItemInfo(fileRepoPath1.getParent());
    }

    @Test
    public void testAddDifferentParentTest() {

        RepoPath fileRepoPath1 = new RepoPathImpl(LIBS_RELEASES_LOCAL, "radiohead/kid_a.txt");
        RepoPath fileRepoPath2 = new RepoPathImpl(LIBS_RELEASES_LOCAL, "tool/10000_days.txt");

        when(repositoryService.getItemInfo(fileRepoPath1.getParent())).thenReturn(itemInfo);
        when(repositoryService.getItemInfo(fileRepoPath2.getParent())).thenReturn(itemInfo);
        when(itemInfo.isFolder()).thenReturn(true);
        folderPruningService.prune(fileRepoPath1.getParent());
        folderPruningService.prune(fileRepoPath2.getParent());
        folderPruningService.doPrune();
        verify(repositoryService, times(1)).getItemInfo(fileRepoPath1.getParent());
        verify(repositoryService, times(1)).getItemInfo(fileRepoPath2.getParent());
    }
}