package org.artifactory.util;

import lombok.experimental.UtilityClass;
import org.artifactory.api.security.AuthorizationService;
import org.artifactory.descriptor.repo.RepoDescriptor;
import org.artifactory.repo.RepoDetails;
import org.artifactory.repo.RepoDetailsType;

import java.util.List;

/**
 * @author Alexei Vainshtein
 */
@UtilityClass
public class RepositoriesUtils {

    /**
     * Adds a list of local, virtual or distribution repositories to the repository details list
     *
     * @param detailsList List that details should be appended to
     * @param reposToAdd  List of repositories to add details of
     * @param type        Type of repository which is being added
     */
    public static void addNonRemoteRepoDetails(String contextUrl, List<RepoDetails> detailsList,
            List<? extends RepoDescriptor> reposToAdd,
            RepoDetailsType type, AuthorizationService authorizationService) {
        for (RepoDescriptor repoToAdd : reposToAdd) {
            String key = repoToAdd.getKey();
            if (authorizationService.userHasPermissionsOnRepositoryRoot(key)) {
                RepoDetails repoDetails = RepoDetails.builder()
                        .key(key)
                        .description(repoToAdd.getDescription())
                        .type(type)
                        .url(contextUrl + "/" + key)
                        .packageType(repoToAdd.getType())
                        .build();
                detailsList.add(repoDetails);
            }
        }
    }
}
