package org.artifactory.util;

import org.artifactory.api.security.AuthorizationService;
import org.artifactory.descriptor.repo.LocalRepoDescriptor;
import org.artifactory.descriptor.repo.RepoType;
import org.artifactory.repo.RepoDetails;
import org.artifactory.repo.RepoDetailsType;
import org.testng.annotations.Test;

import java.util.ArrayList;
import java.util.List;

import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;
import static org.testng.Assert.assertEquals;

/**
 * @author Alexei Vainshtein
 */
public class RepositoriesUtilsTest {

    @Test
    public void testAddNonRemoteRepoDetails() {
        String contextUrl = "url";
        List<RepoDetails> detailsList = new ArrayList<>();
        List<LocalRepoDescriptor> reposToAdd = new ArrayList<>();
        addRepo(reposToAdd, "key", RepoType.Generic);

        addRepo(reposToAdd, "key2", RepoType.Docker);
        AuthorizationService service = mock(AuthorizationService.class);
        when(service.userHasPermissionsOnRepositoryRoot("key")).thenReturn(true);
        when(service.userHasPermissionsOnRepositoryRoot("key2")).thenReturn(false);
        RepositoriesUtils
                .addNonRemoteRepoDetails(contextUrl, detailsList, reposToAdd, RepoDetailsType.FEDERATED, service);
        assertEquals(detailsList.size(), 1);
        assertEquals(detailsList.get(0).getPackageType(), RepoType.Generic);
        assertEquals(detailsList.get(0).getType(), RepoDetailsType.FEDERATED);
        assertEquals(detailsList.get(0).getUrl(), contextUrl + "/key");
    }

    private void addRepo(List<LocalRepoDescriptor> reposToAdd, String key, RepoType type) {
        LocalRepoDescriptor repo2 = new LocalRepoDescriptor();
        repo2.setKey(key);
        repo2.setType(type);
        reposToAdd.add(repo2);
    }
}