package org.artifactory.storage.db;

import org.artifactory.aol.AolService;
import org.artifactory.api.context.ArtifactoryContext;
import org.artifactory.api.context.ArtifactoryContextThreadBinder;
import org.artifactory.storage.db.version.ArtifactoryDBVersion;
import org.artifactory.storage.jobs.async.database.DatabaseConverterRunnable;
import org.artifactory.test.ArtifactoryHomeBoundTest;
import org.artifactory.version.ArtifactoryServiceVersionProvider;
import org.artifactory.version.CompoundVersionDetails;
import org.mockito.Mockito;
import org.mockito.invocation.InvocationOnMock;
import org.testng.Assert;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.Test;

import javax.validation.constraints.NotNull;
import java.util.List;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;

import static org.jfrog.converter.model.ConverterExecutionType.SEQ_SYNC;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.when;

/**
 * @author Alexei Vainshtein
 */
public class AsyncDatabaseConverterAdapterTest extends ArtifactoryHomeBoundTest {

    @AfterMethod
    public void tearDown() {
        ArtifactoryContextThreadBinder.unbind();
    }

    @Test
    public void testConvert() {
        ArtifactoryContext context = Mockito.mock(ArtifactoryContext.class);
        createAolService(context, false);

        AsyncDatabaseConverterAdapter converterAdapter =
                Mockito.spy(new AsyncDatabaseConverterAdapter(homeStub));
        Mockito.doReturn(List.of(ArtifactoryDBVersion.get(SEQ_SYNC))).
                when(converterAdapter).getDbVersions(any(), any());
        CompoundVersionDetails source = Mockito.mock(CompoundVersionDetails.class);
        when(source.getVersion()).thenReturn(ArtifactoryServiceVersionProvider.v790m001.get());
        CompoundVersionDetails target = Mockito.mock(CompoundVersionDetails.class);
        ScheduledExecutorService executor =
                Mockito.mock(TestArtifactoryExecutor.class, InvocationOnMock::callRealMethod);
        when(converterAdapter.getScheduledExecutorService()).thenReturn(executor);
        converterAdapter.convert(source, target);
        Mockito.verify(executor, Mockito.times(1)).shutdown();

        when(context.beanForType(org.artifactory.storage.db.util.JdbcHelper.class))
                .thenReturn(Mockito.mock(org.artifactory.storage.db.util.JdbcHelper.class));
        ArtifactoryContextThreadBinder.bind(context);
    }

    @Test
    public void testGetMinutesDelay() {
        AsyncDatabaseConverterAdapter converterAdapter = Mockito.spy(
                new AsyncDatabaseConverterAdapter(homeStub));
        ArtifactoryContext context = Mockito.mock(ArtifactoryContext.class);
        AolService aolService = createAolService(context, true);

        int minutesDelay = converterAdapter.getMinutesDelay();
        Assert.assertTrue(minutesDelay > 0);

        when(aolService.isAol()).thenReturn(false);
        minutesDelay = converterAdapter.getMinutesDelay();
        Assert.assertEquals(minutesDelay, 0);
    }

    @NotNull
    private AolService createAolService(ArtifactoryContext context, boolean isAol) {
        AolService aolService = Mockito.mock(AolService.class);
        when(aolService.isAol()).thenReturn(isAol);
        ArtifactoryContextThreadBinder.bind(context);
        when(context.beanForType(AolService.class)).thenReturn(aolService);
        return aolService;
    }

    private static abstract class TestArtifactoryExecutor implements ScheduledExecutorService {

        @NotNull
        @Override
        public ScheduledFuture<?> schedule(@NotNull Runnable command, long delay,
                @NotNull TimeUnit unit) {
            Assert.assertTrue(command instanceof DatabaseConverterRunnable);
            return null;
        }

        @Override
        public void shutdown() {

        }
    }
}