package org.artifactory.storage.db.aql.sql.builder.query.aql;

import org.artifactory.aql.model.AqlDomainEnum;
import org.artifactory.aql.model.AqlField;
import org.artifactory.aql.model.AqlPhysicalFieldEnum;
import org.artifactory.aql.model.AqlValue;
import org.artifactory.aql.model.AqlVariableTypeEnum;
import org.artifactory.storage.db.aql.sql.builder.query.sql.SqlTable;
import org.artifactory.storage.db.aql.sql.model.SqlTableEnum;
import org.testng.annotations.Test;

import java.util.ArrayList;
import java.util.List;

import static org.testng.Assert.assertEquals;

public class SimplePropertyCriterionSubSelectTest {

    @Test
    public void testNotMatchFromNodesToNodeProps() {
        Criterion newCriterion =
                new SimplePropertyCriterionSubSelect(List.of(AqlDomainEnum.items, AqlDomainEnum.properties),
                        new AqlField(AqlPhysicalFieldEnum.propertyKey), new SqlTable(SqlTableEnum.node_props, 100),
                        "$nmatch",
                        new AqlValue(AqlVariableTypeEnum.string, "value1"),
                        new SqlTable(SqlTableEnum.node_props, 100), false);
        String expectedValue =
                "( n.node_id not in ( select npsub.node_id " +
                        "from node_props npsub " +
                        "where npsub.node_id = n.node_id and npsub.prop_key like ? ))";
        ArrayList param = new ArrayList();
        param.add("value1");
        assertEquals(newCriterion.toSql(param), expectedValue);
    }

    @Test
    public void testNotEqualFromNodesToNodeProps() {
        Criterion newCriterion =
                new SimplePropertyCriterionSubSelect(List.of(AqlDomainEnum.items, AqlDomainEnum.properties),
                        new AqlField(AqlPhysicalFieldEnum.propertyKey), new SqlTable(SqlTableEnum.node_props, 100),
                        "$ne",
                        new AqlValue(AqlVariableTypeEnum.string, "*"),
                        new SqlTable(SqlTableEnum.node_props, 100), false);
        String expectedValue =
                "( n.node_id not in ( select npsub.node_id " +
                        "from node_props npsub " +
                        "where npsub.node_id = n.node_id and npsub.prop_key = ? ))";
        assertEquals(newCriterion.toSql(new ArrayList()), expectedValue);
    }

    @Test
    public void testNotEqualFromBuildModulesToBuildProps() {
        Criterion newCriterion =
                new SimplePropertyCriterionSubSelect(List.of(AqlDomainEnum.modules, AqlDomainEnum.builds,
                        AqlDomainEnum.buildProperties),
                        new AqlField(AqlPhysicalFieldEnum.propertyKey), new SqlTable(SqlTableEnum.build_props, 100),
                        "$ne",
                        new AqlValue(AqlVariableTypeEnum.string, "*"),
                        new SqlTable(SqlTableEnum.build_props, 100), false);
        String expectedValue =
                "( b.build_id not in ( select bpsub.build_id " +
                        "from build_props bpsub " +
                        "where bpsub.build_id = b.build_id and bpsub.prop_key = ? ))";
        assertEquals(newCriterion.toSql(new ArrayList()), expectedValue);
    }

    @Test
    public void testNotMatchFromBuildModulesToBuildProps() {
        Criterion newCriterion =
                new SimplePropertyCriterionSubSelect(List.of(AqlDomainEnum.modules, AqlDomainEnum.builds,
                        AqlDomainEnum.buildProperties),
                        new AqlField(AqlPhysicalFieldEnum.propertyKey), new SqlTable(SqlTableEnum.build_props, 100),
                        "$nmatch",
                        new AqlValue(AqlVariableTypeEnum.string, "*"),
                        new SqlTable(SqlTableEnum.build_props, 100), false);
        String expectedValue =
                "( b.build_id not in ( select bpsub.build_id " +
                        "from build_props bpsub " +
                        "where bpsub.build_id = b.build_id and bpsub.prop_key like ? ))";
        assertEquals(newCriterion.toSql(new ArrayList()), expectedValue);
    }
}
