package org.artifactory.storage.db.build.dao;

import org.artifactory.api.build.PublishedModule;
import org.testng.annotations.DataProvider;
import org.testng.annotations.Test;

import java.sql.ResultSet;
import java.sql.SQLException;

import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;
import static org.testng.Assert.assertEquals;

public class BuildsDaoTest {

    @DataProvider(name = "resultSets")
    public static Object[][] resultSets() {
        return new Object[][] {
                {"id", "0", "1", null,
                        pm(null, null, null, "id", "0", "1")
                },
                {"id/1", "0", "1", "build",
                        pm("build", "id", "1", "id/1", "0", "1")
                },
                {"id/1", "0", "1", "Build",
                        pm("build", "id", "1", "id/1", "0", "1")
                },
                {"id", "0", "1", "Build",
                        pm("build", "", "", "id", "0", "1")
                },
                {"id/", "0", "1", "Build",
                        pm("build", "id", "", "id/", "0", "1")
                },
                {"/", "0", "1", "Build",
                        pm("build", "", "", "/", "0", "1")
                },
                {"/2", "0", "1", "Build",
                        pm("build", "", "2", "/2", "0", "1")
                },
                {"", "0", "1", "Build",
                        pm("build", "", "", "", "0", "1")
                },
                {"", "0", "1", "Ivy",
                        pm("Ivy", null, null, "", "0", "1")
                }
        };
    }

    @Test(dataProvider = "resultSets")
    public void testGetPublishedModule(String id, String artifacts, String dependencies, String type,
            PublishedModule expected) throws SQLException {
        BuildsDao buildsDao = new BuildsDao(null, null, null);
        ResultSet resultSet = mock(ResultSet.class);
        when(resultSet.getString(1)).thenReturn(id);
        when(resultSet.getString(2)).thenReturn(artifacts);
        when(resultSet.getString(3)).thenReturn(dependencies);
        when(resultSet.getString(4)).thenReturn(type);
        PublishedModule module = buildsDao.getPublishedModule(resultSet);
        assertEquals(module, expected);
    }

    private static PublishedModule pm(String type, String buildId, String buildNumber, String id, String numOfArtifact,
            String numOfDependencies) {
        PublishedModule pm = new PublishedModule();
        pm.setType(type);
        pm.setBuildId(buildId);
        pm.setBuildNumber(buildNumber);
        pm.setId(id);
        pm.setNumOfArtifact(numOfArtifact);
        pm.setNumOfDependencies(numOfDependencies);
        return pm;
    }
}