package org.artifactory.storage.db.event.itest.service;

import org.artifactory.event.EventType;
import org.artifactory.storage.db.event.dao.EventsDao;
import org.artifactory.storage.db.itest.DbBaseTest;
import org.artifactory.storage.event.EventInfo;
import org.artifactory.storage.event.EventsService;
import org.jfrog.common.ClockUtils;
import org.mockito.internal.util.collections.Sets;
import org.springframework.beans.factory.annotation.Autowired;
import org.testng.Assert;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import java.util.Arrays;
import java.util.List;
import java.util.Set;

public class EventsServiceImplShiftEventsITest extends DbBaseTest {

    @Autowired
    private EventsService eventsService;
    @Autowired
    private EventsDao eventsDao;

    @BeforeMethod
    private void setup() throws Exception{
        importSql("/sql/events.sql", false);
        assertCount("node_events_tmp", 18);
        assertCount("node_events", 0);
    }

    @AfterMethod
    private void clean() throws Exception{
        eventsDao.deleteAll();
    }

    @Test
    public void testShiftEventsTx() throws Exception {
        // Verify the shift copied and delete the tmp events in one TX
        Set<String> repos = eventsService.shiftEvents();
        Assert.assertNotNull(repos);
        Assert.assertEquals(repos.size(), 3);
        Assert.assertEquals(repos, Sets.newSet("repo1", "repo2", "repo-no-create"));
        assertCount("node_events_tmp", 0);
        assertCount("node_events", 18);
    }

    @Test
    public void testAppendEvents() throws Exception{
        List<EventInfo> tmpEvents = Arrays.asList (
                new EventInfo(123, EventType.create, "/repo1/a", Sets.newSet("hints1")),
                new EventInfo(123, EventType.delete, "/repo1/b", Sets.newSet("hints2"))
        );
        Thread.sleep(10);
        long timestampBeforeAppend = ClockUtils.epochMillis();
        Thread.sleep(10);
        eventsService.appendEvents(tmpEvents);

        assertCount("node_events_tmp", 20);

        Thread.sleep(10);
        long timestampBeforeShift = ClockUtils.epochMillis();
        Thread.sleep(10);
        eventsService.shiftEvents();

        //Verify that new timestamp was added to each event while inserted to node_events_tmp
        List<EventInfo> events = eventsService.getEventsAfter(timestampBeforeAppend, "repo1");
        events.forEach(event -> Assert.assertTrue(event.getTimestamp() > timestampBeforeAppend));
        events.forEach(event -> Assert.assertTrue(event.getTimestamp() < timestampBeforeShift));

    }

    private void assertCount(String table, int expected) throws Exception {
        Assert.assertEquals(jdbcHelper.executeSelectCount("SELECT COUNT(1) FROM " + table), expected);
    }
}
