package org.artifactory.storage.db.upgrades.common;

import org.artifactory.aol.AolService;
import org.artifactory.api.context.ArtifactoryContext;
import org.artifactory.api.context.ArtifactoryContextThreadBinder;
import org.artifactory.common.home.ArtifactoryHome;
import org.artifactory.opentracing.ArtifactoryTracer;
import org.artifactory.storage.db.init.CommonDataSourceFactory;
import org.artifactory.storage.db.util.JdbcHelper;
import org.artifactory.version.ArtifactoryServiceVersion;
import org.artifactory.version.CompoundVersionDetails;
import org.testng.Assert;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;

import java.util.concurrent.TimeUnit;

import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

/**
 * @author Alexei Vainshtein
 */
public abstract class AsyncUpgradeBaseTest extends UpgradeBaseTest {

    private DatabaseTestConverterAdapter asyncDatabaseConverterAdapter;

    @BeforeClass
    @Override
    protected void springTestContextPrepareTestInstance() throws Exception {
        super.springTestContextPrepareTestInstance();
        asyncDatabaseConverterAdapter = new DatabaseTestConverterAdapter(ArtifactoryHome.get());
        runAsyncConversion();
        waitForAsyncToFinish();
    }

    protected void runAsyncConversion() {
        ArtifactoryContext context = mock(ArtifactoryContext.class);
        ArtifactoryTracer tracer = mock(ArtifactoryTracer.class);
        when(tracer.getCurrentThreadActiveSpan()).thenReturn(null);
        when(context.beanForType(ArtifactoryTracer.class)).thenReturn(tracer);
        AolService aolService = mock(AolService.class);
        when(aolService.isAol()).thenReturn(false);
        when(context.beanForType(AolService.class)).thenReturn(aolService);
        when(context.beanForType(org.jfrog.storage.JdbcHelper.class))
                .thenReturn(new JdbcHelper(CommonDataSourceFactory.createDataSource(dbProperties, false)));
        when(context.getArtifactoryHome()).thenReturn(ArtifactoryHome.get());
        ArtifactoryContextThreadBinder.bind(context);
        final CompoundVersionDetails source =
                new CompoundVersionDetails(ArtifactoryServiceVersion.getCurrent(), "", System.currentTimeMillis());
        asyncDatabaseConverterAdapter.convert(source, source);
    }

    @AfterClass
    public void tearDown() {
        ArtifactoryContextThreadBinder.unbind();
    }

    // Waits for 5 seconds to the async to finish.
    // If not finished after 5 seconds, fail the test
    protected void waitForAsyncToFinish() throws InterruptedException {
        if (!asyncDatabaseConverterAdapter.getScheduledExecutorService().awaitTermination(10, TimeUnit.SECONDS)) {
            Assert.fail("Executor wasn't terminated after 5 seconds. Failing the test.");
        }
    }
}
