package org.artifactory.ui.rest.service.artifacts.search.remotesearch;

import org.artifactory.api.bintray.BintrayItemInfo;
import org.artifactory.api.bintray.BintrayService;
import org.artifactory.api.repo.RepositoryService;
import org.artifactory.api.search.BintrayItemSearchResults;
import org.artifactory.api.search.remote.RemoteSearchControls;
import org.artifactory.projects.ProjectsService;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;

import java.util.List;

class RemoteSearchServiceTest {

    private static final String PROJ_REMOTE_REPO = "proj-remote";

    private RemoteSearchService remoteSearchService;
    @Mock
    private BintrayService bintrayService;
    @Mock
    private ProjectsService projectsService;
    @Mock
    private RepositoryService repoService;

    private AutoCloseable autoCloseable;

    @BeforeEach
    public void setUp() throws Exception {
        autoCloseable = MockitoAnnotations.openMocks(this);
        remoteSearchService = Mockito.spy(new RemoteSearchService(bintrayService, projectsService, repoService));
    }

    @Test
    void testLimitResultToProjectLocalReposWithProjectSearchControl() {
        RemoteSearchControls remoteSearchControls = new RemoteSearchControls();
        remoteSearchControls.setSelectedRepoForSearch(List.of(PROJ_REMOTE_REPO));
        BintrayItemSearchResults<BintrayItemInfo> bintrayItemSearchResults = createBintrayItemSearchResults();

        BintrayItemSearchResults<BintrayItemInfo> limitedResults =
                remoteSearchService.limitResultToProjectLocalRepos(bintrayItemSearchResults, remoteSearchControls);
        Assertions.assertNotNull(limitedResults);
        Assertions.assertEquals(0, limitedResults.getResults().size());
    }

    @Test
    void testLimitResultToProjectLocalReposWithEmptySearchControl() {
        RemoteSearchControls remoteSearchControls = new RemoteSearchControls();
        BintrayItemSearchResults<BintrayItemInfo> bintrayItemSearchResults = createBintrayItemSearchResults();

        BintrayItemSearchResults<BintrayItemInfo> limitedResults =
                remoteSearchService.limitResultToProjectLocalRepos(bintrayItemSearchResults, remoteSearchControls);
        Assertions.assertNotNull(limitedResults);
        Assertions.assertEquals(1, limitedResults.getResults().size());
    }

    private BintrayItemSearchResults<BintrayItemInfo> createBintrayItemSearchResults() {
        BintrayItemInfo itemInfo = Mockito.mock(BintrayItemInfo.class);
        Mockito.when(itemInfo.getLocalRepoPath()).thenReturn(null);
        Mockito.when(itemInfo.getName()).thenReturn("abc.jar");
        return new BintrayItemSearchResults<>(List.of(itemInfo), 1);
    }

    @AfterEach
    void tearDown() throws Exception {
        if (autoCloseable != null) {
            autoCloseable.close();
            autoCloseable = null;
        }
    }

}